/*********************************************************/
/* This source code copyright (c) 1991-2001, Aaron Giles */
/* See the Read Me file for licensing information.       */
/* Contact email: mac@aarongiles.com                     */
/*********************************************************/

#ifndef __SLIDECONTROLSWINDOW__
#define __SLIDECONTROLSWINDOW__

enum {
	standard = 0,
	depressed = 1,
	disabled = 2
};

enum {
	kReverseButton = 0,
	kStopButton = 1,
	kPauseButton = 2,
	kForwardButton = 3,
	kHideButton = 4
};

extern OSErr OpenSlideControls(void);
extern void CloseSlideControls(void);
extern WindowPtr GetSlideControlsWindow(void);
extern void SetSlideControlButton(short id, short status);
extern short GetSlideControlButton(short id);
extern void DrawSlideControlsWindow(void);
extern void HandleSlideControlsClick(Point where, short modifiers, Boolean actNow);
extern void HandleSlideControlsKey(char theChar, short modifiers, Boolean actNow);
extern Boolean HandleUserControlButton(short button);
extern Boolean HandleAutomaticButton(short button);
extern Boolean PauseSlideShow(Boolean *advance);
extern Boolean HideSlideShow(Boolean *advance);
extern void SetSlideControlsText(StringPtr string0, StringPtr string1, StringPtr string2,
	StringPtr string3);
extern void DoSlideControlsHelp(Point globalPt);

#endif